import 'call_stack.dart';
import 'path.dart';
import 'runtime_object.dart';

/// A generated Choice from the story.
/// A single ChoicePoint in the Story could potentially generate
/// different Choices dynamically dependent on state, so they're
/// separated.
class Choice extends RuntimeObject {
  /// The main text to presented to the player for this Choice.
  late String text;

  /// The target path that the Story should be diverted to if
  /// this Choice is chosen.
  String get pathStringOnChoice => targetPath.toString();

  set pathStringOnChoice(String value) => targetPath = Path(value);

  /// Get the path to the original choice point - where was this choice defined in the story?
  /// [sourcePath] : A dot separated path into the story data.
  String sourcePath = '';

  /// The original index into currentChoices list on the [StoryInterface] when
  /// this [Choice] was generated, for convenience.
  @override
  int index = 0;

  Path? targetPath;

  Thread? threadAtGeneration;

  int originalThreadIndex = 0;

  bool isInvisibleDefault = false;

  List<String> tags = [];

  static Choice fromJson(Map<String, dynamic> json) {
    var choice = Choice();
    choice.text = json['text'].toString();
    choice.index = json['index'] as int;
    choice.sourcePath = json['originalChoicePath'].toString();
    choice.originalThreadIndex = json['originalThreadIndex'] as int;
    choice.pathStringOnChoice = json['targetPath'].toString();
    return choice;
  }

  Map<String, dynamic> toJson() {
    return <String, dynamic>{
      'text': text,
      'index': index,
      'originalChoicePath': sourcePath,
      'originalThreadIndex': originalThreadIndex,
      'targetPath': pathStringOnChoice
    };
  }
}
