import 'container.dart';
import 'path.dart';
import 'runtime_object.dart';

class ChoicePoint extends RuntimeObject {
  Path? _pathOnChoice;

  Path get pathOnChoice {
    // Resolve any relative paths to global ones as we come across them
    if (_pathOnChoice != null && _pathOnChoice!.isRelative) {
      var choiceTargetObj = choiceTarget;
      if (choiceTargetObj != null) {
        _pathOnChoice = choiceTargetObj.path;
      }
    }
    return _pathOnChoice!;
  }

  set pathOnChoice(Path v) => _pathOnChoice = v;

  Container? get choiceTarget => resolvePath(_pathOnChoice!).container;

  String get pathStringOnChoice => compactPathString(pathOnChoice);

  set pathStringOnChoice(String v) => pathOnChoice = Path(v);

  bool hasCondition = false;
  bool hasStartContent = false;
  bool hasChoiceOnlyContent = false;
  bool onceOnly;
  bool isInvisibleDefault = false;

  int get flags {
    var flags = 0;
    if (hasCondition) flags |= 1;
    if (hasCondition) flags |= 1;
    if (hasStartContent) flags |= 2;
    if (hasChoiceOnlyContent) flags |= 4;
    if (isInvisibleDefault) flags |= 8;
    if (onceOnly) flags |= 16;
    return flags;
  }

  set flags(value) {
    hasCondition = (value & 1) > 0;
    hasStartContent = (value & 2) > 0;
    hasChoiceOnlyContent = (value & 4) > 0;
    isInvisibleDefault = (value & 8) > 0;
    onceOnly = (value & 16) > 0;
  }

  ChoicePoint([this.onceOnly = true]);

  @override
  String toString() {
    var targetLineNum = debugLineNumberOfPath(pathOnChoice);
    var targetString = pathOnChoice.toString();

    if (targetLineNum != null) {
      targetString = ' line $targetLineNum ($targetString)';
    }

    return 'Choice: -> $targetString';
  }
}
