import 'runtime_object.dart';

enum CommandType {
  NotSet, // -1
  EvalStart,
  EvalOutput,
  EvalEnd,
  Duplicate,
  PopEvaluatedValue,
  PopFunction,
  PopTunnel,
  BeginString,
  EndString,
  NoOp,
  ChoiceCount,
  Turns,
  TurnsSince,
  ReadCount,
  Random,
  SeedRandom,
  VisitIndex,
  SequenceShuffleIndex,
  StartThread,
  Done,
  End,
  ListFromInt,
  ListRange,
  ListRandom,
  BeginTag,
  EndTag,
//----
  TOTAL_VALUES;
}

class ControlCommand extends RuntimeObject {
  CommandType commandType;

  ControlCommand([this.commandType = CommandType.NotSet]);

  ControlCommand copy() {
    return ControlCommand(commandType);
  }

// The following static factory methods are to make generating these objects
// slightly more succinct. Without these, the code gets pretty massive! e.g.
//
//     var c = new Runtime.ControlCommand(Runtime.ControlCommand.CommandType.EvalStart)
//
// as opposed to
//
//     var c = Runtime.ControlCommand.EvalStart()

  static ControlCommand EvalStart() {
    return ControlCommand(CommandType.EvalStart);
  }

  static ControlCommand EvalOutput() {
    return ControlCommand(CommandType.EvalOutput);
  }

  static ControlCommand EvalEnd() {
    return ControlCommand(CommandType.EvalEnd);
  }

  static ControlCommand Duplicate() {
    return ControlCommand(CommandType.Duplicate);
  }

  static ControlCommand PopEvaluatedValue() {
    return ControlCommand(CommandType.PopEvaluatedValue);
  }

  static ControlCommand PopFunction() {
    return ControlCommand(CommandType.PopFunction);
  }

  static ControlCommand PopTunnel() {
    return ControlCommand(CommandType.PopTunnel);
  }

  static ControlCommand BeginString() {
    return ControlCommand(CommandType.BeginString);
  }

  static ControlCommand EndString() {
    return ControlCommand(CommandType.EndString);
  }

  static ControlCommand NoOp() {
    return ControlCommand(CommandType.NoOp);
  }

  static ControlCommand ChoiceCount() {
    return ControlCommand(CommandType.ChoiceCount);
  }

  static ControlCommand Turns() {
    return ControlCommand(CommandType.Turns);
  }

  static ControlCommand TurnsSince() {
    return ControlCommand(CommandType.TurnsSince);
  }

  static ControlCommand ReadCount() {
    return ControlCommand(CommandType.ReadCount);
  }

  static ControlCommand Random() {
    return ControlCommand(CommandType.Random);
  }

  static ControlCommand SeedRandom() {
    return ControlCommand(CommandType.SeedRandom);
  }

  static ControlCommand VisitIndex() {
    return ControlCommand(CommandType.VisitIndex);
  }

  static ControlCommand SequenceShuffleIndex() {
    return ControlCommand(CommandType.SequenceShuffleIndex);
  }

  static ControlCommand StartThread() {
    return ControlCommand(CommandType.StartThread);
  }

  static ControlCommand Done() {
    return ControlCommand(CommandType.Done);
  }

  static ControlCommand End() {
    return ControlCommand(CommandType.End);
  }

  static ControlCommand ListFromInt() {
    return ControlCommand(CommandType.ListFromInt);
  }

  static ControlCommand ListRange() {
    return ControlCommand(CommandType.ListRange);
  }

  static ControlCommand ListRandom() {
    return ControlCommand(CommandType.ListRandom);
  }
  static ControlCommand BeginTag() {
    return ControlCommand(CommandType.BeginTag);
  }

  static ControlCommand EndTag() {
    return ControlCommand(CommandType.EndTag);
  }


  @override
  String toString() {
    return commandType.toString();
  }
}
