

import 'call_stack.dart';
import 'choice.dart';
import 'json.dart';
import 'runtime_object.dart';
import 'simple_json.dart';
import 'story.dart';

class Flow {
  String name;
  CallStack callStack;
  late List<RuntimeObject> outputStream;
  late List<Choice> currentChoices;

  Flow(this.name, Story story, [Map<String, dynamic>? jObject])
      : callStack = CallStack.of(storyContext: story) {
    if (jObject != null) {
      callStack.setJsonToken(
          jObject['callstack'] as Map<String, dynamic>, story);
      outputStream = JsonSerialization.JArrayToRuntimeObjList(
          jObject['outputStream'] as List<dynamic>);
      currentChoices = JsonSerialization.JArrayToRuntimeObjList<Choice>(
          jObject['currentChoices'] as List<dynamic>);

      // choiceThreads is optional
      dynamic jChoiceThreadsObj = jObject['choiceThreads'];
      if (jChoiceThreadsObj != null) {
        LoadFlowChoiceThreads(jObject['choiceThreads'], story);
      }
    } else {
      outputStream = [];
      currentChoices = [];
    }
  }

  void WriteJson(SimpleJsonWriter writer) {
    writer.WriteObjectStart();

    writer.WriteProperty("callstack", (w) => callStack.WriteJson(w));
    writer.WriteProperty("outputStream",
        (w) => JsonSerialization.WriteListRuntimeObjs(w, outputStream));

    // choiceThreads: optional
    // Has to come BEFORE the choices themselves are written out
    // since the originalThreadIndex of each choice needs to be set
    bool hasChoiceThreads = false;
    for (Choice c in currentChoices) {
      c.originalThreadIndex = c.threadAtGeneration!.threadIndex;

      if (callStack.threadWithIndex(c.originalThreadIndex) == null) {
        if (!hasChoiceThreads) {
          hasChoiceThreads = true;
          writer.WritePropertyStart("choiceThreads");
          writer.WriteObjectStart();
        }

        writer.WritePropertyStart(c.originalThreadIndex);
        c.threadAtGeneration!.WriteJson(writer);
        writer.WritePropertyEnd();
      }
    }

    if (hasChoiceThreads) {
      writer.WriteObjectEnd();
      writer.WritePropertyEnd();
    }

    writer.WriteProperty("currentChoices", (w) {
      w.WriteArrayStart();
      for (var c in currentChoices) {
        JsonSerialization.WriteChoice(w, c);
      }
      w.WriteArrayEnd();
    });

    writer.WriteObjectEnd();
  }

  // Used both to load old format and current
  void LoadFlowChoiceThreads(Map<String, dynamic> jChoiceThreads, Story story) {
    for (final choice in currentChoices) {
      var foundActiveThread =
          callStack.threadWithIndex(choice.originalThreadIndex);
      if (foundActiveThread != null) {
        choice.threadAtGeneration = foundActiveThread.copy();
      } else {
        Map<String, dynamic> jSavedChoiceThread =
            jChoiceThreads[choice.originalThreadIndex.toString()];
        //if (jSavedChoiceThread != null) {
        choice.threadAtGeneration = Thread.from(jSavedChoiceThread, story);
        //}
      }
    }
  }
}
