import 'package:async/async.dart';
import 'ink_list.dart';

class ListDefinition {
  final String _name;
  // The main representation should be simple item names rather than a RawListItem,
  // since we mainly want to access items based on their simple name, since that's
  // how they'll be most commonly requested from ink.
  final Map<String, int> _itemNameToValues;
  ListDefinition(this._name, this._itemNameToValues);

  Map<InkListItem, int>? _items;

  String get name => _name;

  //@late
  Map<InkListItem, int> get items {
    if (_items == null) {
      _items = {};
      for (var itemNameAndValue in _itemNameToValues.entries) {
        var item =
            InkListItem(originName: name, itemName: itemNameAndValue.key);
        _items![item] = itemNameAndValue.value;
      }
    }
    return _items!;
  }

  int valueForItem(InkListItem item) {
    return _itemNameToValues[item.itemName] ?? 0;
  }

  bool containsItem(InkListItem item) {
    if (item.originName != null) {
      return false;
    } else {
      return _itemNameToValues.containsKey(item.itemName);
    }
  }

  bool containsItemWithName(String itemName) {
    return _itemNameToValues.containsKey(itemName);
  }

  Result<InkListItem> getItemWithValue(int val) {
    for (var namedItem in _itemNameToValues.entries) {
      if (namedItem.value == val) {
        return Result.value(
            InkListItem(originName: name, itemName: namedItem.key));
      }
    }
    return Result.error(InkListItem.Null);
  }

  int? getValueForItem(InkListItem item) => _itemNameToValues[item.itemName];
}
