import 'dart:async';

import 'story.dart';
import 'package:logging/logging.dart' as $logging;
import 'package:meta/meta.dart';

@internal
class Logger implements $logging.Logger {
  final $logging.Logger _delegate;

  Logger(String name) : _delegate = $logging.Logger(name);

  @override
  $logging.Level get level => _delegate.level;

  @override
  Map<String, $logging.Logger> get children => _delegate.children;

  @override
  void clearListeners() => _delegate.clearListeners();

  @override
  void config(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.CONFIG, message, error, stackTrace);

  @override
  void fine(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.FINE, message, error, stackTrace);

  @override
  void finer(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.FINER, error, stackTrace);

  @override
  void finest(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.FINEST, message, error, stackTrace);

  @override
  String get fullName => _delegate.fullName;

  @override
  void info(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.INFO, message, error, stackTrace);

  @override
  bool isLoggable($logging.Level value) =>
      Story.debug && _delegate.isLoggable(value);

  @override
  void log($logging.Level logLevel, Object? message,
      [Object? error, StackTrace? stackTrace, Zone? zone]) {
    if (isLoggable(logLevel)) {
      _delegate.log(logLevel, message, error, stackTrace, zone);
    }
  }

  @override
  String get name => _delegate.name;

  @override
  Stream<$logging.LogRecord> get onRecord => Stream.empty();

  @override
  $logging.Logger? get parent => _delegate.parent;

  @override
  void severe(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.SEVERE, message, error, stackTrace);

  @override
  void shout(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.SHOUT, message, error, stackTrace);

  @override
  void warning(Object? message, [Object? error, StackTrace? stackTrace]) =>
      log($logging.Level.WARNING, message, error, stackTrace);

  @override
  set level($logging.Level? value) => _delegate.level = value;
}
