import 'container.dart';
import 'logger.dart';
import 'path.dart';
import 'runtime_object.dart';

final _logger = Logger('Pointer');

/// Internal structure used to point to a particular / current point in the story.
/// Where Path is a set of components that make content fully addressable, this is
/// a reference to the current container, and the index of the current piece of
/// content within that container. This scheme makes it as fast and efficient as
/// possible to increment the pointer (move the story forwards) in a way that's as
/// native to the internal engine as possible.
class Pointer {
  Container? container;
  int index = -1;

  Pointer({this.container, this.index = -1});

  factory Pointer.startOf(Container? container) {
    return Pointer(container: container, index: 0);
  }

  //void assign(Pointer pointer) {
  //  index = pointer.index;
  //  container = pointer.container;
  //}

  //public Runtime.Object Resolve ()
  RuntimeObject? resolve() {
    _logger.finest('[resolve] ${this}');
    if (index < 0) {
      return container;
    }
    if (container == null) {
      return null;
    }
    if (container!.content.isEmpty) {
      return container;
    }
    if (index >= container!.content.length) {
      return null;
    }
    return container!.content[index];
  }

  static Pointer get Null => Pointer(
        container: null,
      );

  bool get isNull {
    _logger.finest('isNull : ${container == null}');
    return container == null;
  }

  bool get isNotNull => !isNull;

  Path? get path {
    if (isNull) {
      return null;
    }
    if (index >= 0) {
      return container!.path.pathByAppendingComponent(Component.index(index));
    } else {
      return container!.path;
    }
  }

  @override
  String toString() {
    if (container == null) {
      return 'Ink Pointer (null)';
    } else {
      return 'Ink Pointer -> ${container!.path.toString()}  -- index $index';
    }
  }

  //factory Pointer.fromJson(Map<String, dynamic> json) =>
  //    _$PointerFromJson(json);

  // This method does not exist in the original C# code, but is here to maintain the
  // value semantics of Pointer.
  Pointer copyWith({int? index}) {
    return Pointer(container: container, index: index ?? this.index);
  }

//Map<String, dynamic> toJson() => _$PointerToJson(this);
}

String? containerToJson(Container? container) {
  return container?.path.componentsString;
}
