import 'dart:collection';

import 'container.dart';
import 'runtime_object.dart';

class StatePatch {
  late Map<String, RuntimeObject> _globals;
  HashSet<String> _changedVariables = HashSet<String>();
  Map<Container, int> _visitCounts = <Container, int>{};
  Map<Container, int> _turnIndices = <Container, int>{};

  Map<String, RuntimeObject> get globals => _globals;

  HashSet<String> get changedVariables => _changedVariables;

  Map<Container, int> get visitCounts => _visitCounts;

  Map<Container, int> get turnIndices => _turnIndices;

  StatePatch([StatePatch? toCopy]) {
    if (toCopy != null) {
      _globals = Map<String, RuntimeObject>.from(toCopy._globals);
      _changedVariables = HashSet<String>.from(toCopy._changedVariables);
      _visitCounts = Map<Container, int>.from(toCopy._visitCounts);
      _turnIndices = Map<Container, int>.from(toCopy._turnIndices);
    } else {
      _globals = <String, RuntimeObject>{};
      _changedVariables = HashSet<String>();
      _visitCounts = <Container, int>{};
      _turnIndices = <Container, int>{};
    }
  }

  RuntimeObject? getGlobal(String? name) => _globals[name];

  void setGlobal(String name, RuntimeObject value) {
    _globals[name] = value;
  }

  void addChangedVariable(String name) {
    _changedVariables.add(name);
  }

  int? getVisitCount(Container container) {
    return _visitCounts[container];
  }

  void setVisitCount(Container container, int count) {
    _visitCounts[container] = count;
  }

  void setTurnIndex(Container container, int index) {
    _turnIndices[container] = index;
  }

  int? getTurnIndex(Container container) {
    return _turnIndices[container];
  }
}
