import 'package:ink_runtime/ink.dart';
import 'package:logging/logging.dart';
import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  Story story;

  logSetUp();
  group('Diverts', () {
    setUp(() {});

    test('- divert text from one knot/stitch to another', () {
      story = loadStory('divert/simple-divert.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 2);
      expect(text.first, 'We arrived into London at 9.45pm exactly.');
      expect(text.last, 'We hurried home to Savile Row as fast as we could.');
    });

    test(
        'invisibleDivert - divert from one line of text to new content invisibly',
        () {
      story = loadStory('divert/invisible-divert.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'We hurried home to Savile Row as fast as we could.');
    });

    test('divertOnChoice - branch directly from choices', () {
      story = loadStory('divert/divert-on-choice.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'You open the gate, and step out onto the path.');
    });

    test(
        'complexBranching1 - be usable to branch and join text seamlessly (example 1)',
        () {
      story = loadStory('divert/complex-branching.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);

      expect(text.length, 2);
      expect(text[0], "\"There is not a moment to lose!\" I declared.");
      expect(text[1], "We hurried home to Savile Row as fast as we could.");
    });

    test(
        'complexBranching2 - be usable to branch and join text seamlessly (example 2)',
        () {
      story = loadStory('divert/complex-branching.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);

      expect(text.length, 3);
      expect(text[0], "\"Monsieur, let us savour this moment!\" I declared.");
      expect(text[1],
          "My master clouted me firmly around the head and dragged me out of the door.");
      expect(text[2],
          "He insisted that we hurried home to Savile Row as fast as we could.");
    });
  });
}
