import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  Story story;

  logSetUp();
  group('Function Spec', () {
    setUp(() {});

    test('func basic - return a value from a function in a variable expression',
        () {
      story = loadStory('function/func-basic.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'The value of x is 4.4.');
    });

    test('func none - return a value from a function with no parameters', () {
      story = loadStory('function/func-none.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'The value of x is 3.8.');
    });

    test('func inline - handle conditionals in the functions', () {
      story = loadStory('function/func-inline.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'The value of x is 4.4.');
    });

    test('set var func - be able to set a variable as a command', () {
      story = loadStory('function/setvar-func.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'The value is 6.');
    });

    test(
        'complex func 1 - handle conditionals and setting of variables (test 1)',
        () {
      story = loadStory('function/complex-func1.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'The values are 6 and 10.');
    });

    test(
        'complex func 2 - handle conditionals and setting of variables (test 2)',
        () {
      story = loadStory('function/complex-func2.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text.first, 'The values are -1 and 0 and 1.');
    });

    test(
        'complex func 3 - handle conditionals and setting of variables (test 3)',
        () {
      story = loadStory('function/complex-func3.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 1);
      expect(text[0],
          "\"I will pay you 120 reales if you get the goods to their destination. The goods will take up 20 cargo spaces.\"");
    });

    test('random function', () {
      story = loadStory('function/rnd-func.ink.json');
      var text = <String>[];

      nextAll(story, text);

      expect(text.length, 4);
    });

    /*
    test('evaluatingFunctionVariableStateBug ', () {
      story =
          loadStory('function/evaluating-function-variablestate-bug.ink.json');
      var text = <String>[];

      expect("Start\n", story.Continue());
      expect("In tunnel.\n", story.Continue());

      Object funcResult = story.evaluateFunction("function_to_evaluate");
      expect("RIGHT", funcResult as String);

      expect("End\n", story.Continue());
    });*/
  });
}
