import 'package:ink_runtime/ink.dart';
import 'package:logging/logging.dart';
import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  logSetUp();
  group('Gather Spec', () {
    late Story story;

    setUp(() {});
    tearDown(() {
      //story.resetState();
    });
    test('gather basic - gather the flow back together again', () {
      story = loadStory('gather/gather-basic.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);

      expect(text.length, 3);
      expect(text[0], "\"Nothing, Monsieur!\" I replied.");
      expect(text[1], "\"Very good, then.\"");
      expect(text[2], "With that Monsieur Fogg left the room.");
    });

    test('gather chain - form chains of content with multiple gathers', () {
      story = loadStory('gather/gather-chain.ink.json');
      var text = <String>[];

      nextAll(story, text);
      expect(story.currentChoices.length, 3);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0],
          "I did not pause for breath but kept on running. The road could not be much further! Mackie would have the engine running, and then I'd be safe.");
      expect(story.currentChoices.length, 2);
      text.clear();

      story.chooseChoiceIndex(0);

      nextAll(story, text);
      expect(text.length, 2);
      expect(text[0],
          "I reached the road and looked about. And would you believe it?");
      expect(text[1], "The road was empty. Mackie was nowhere to be seen.");
    });

    test(
        'nestedFlow - allow nested options to pop out to a higher level gather',
        () {
      story = loadStory('gather/nested-flow.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      story.chooseChoiceIndex(2);

      text.clear();
      nextAll(story, text);
      expect(text.length, 2);
      expect(text[0], "\"Myself!\"");
      expect(text[1],
          "Mrs. Christie lowered her manuscript a moment. The rest of the writing group sat, open-mouthed.");
    });

    test(
        'deep nesting - gather the flow back together again from arbitrarily deep options',
        () {
      story = loadStory('gather/deep-nesting.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      expect(text.length, 2);
      expect(text[0], "\"...Tell us a tale Captain!\"");
      expect(text[1], "To a man, the crew began to yawn.");
    });

    test(
        'complex flow 1 - offer a compact way to weave and blend text and options (Example 1)',
        () {
      story = loadStory('gather/complex-flow.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(
          text[0], "... but I said nothing and we passed the day in silence.");
    });

    test(
        'complex flow 2 - offer a compact way to weave and blend text and options (Example 2)',
        () {
      story = loadStory('gather/complex-flow.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      expect(text.length, 3);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      expect(text.length, 2);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(text.length, 2);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(text.length, 3);
    });
  });
}
