import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  logSetUp();
  group('Knot Spec', () {
    late Story story;

    setUp(() {});
    tearDown(() {
      story.resetState();
    });
    test('testSingleLine - A single line of plain text in an ink file', () {
      story = loadStory('knot/single-line.ink.json');
      var text = <String>[];

      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0], "Hello, world!");
    });
    test('Multiple lines of plain text in an ink file', () {
      story = loadStory('knot/multi-line.ink.json');
      var text = <String>[];

      nextAll(story, text);
      expect(text.length, 3);
      expect(text[0], "Hello, world!");
      expect(text[1], "Hello?");
      expect(text[2], "Hello, are you there?");
    });

    test('strip empty lines of output', () {
      story = loadStory('knot/strip-empty-lines.ink.json');
      var text = <String>[];

      nextAll(story, text);
      expect(text.length, 3);
      expect(text[0], "Hello, world!");
      expect(text[1], "Hello?");
      expect(text[2], "Hello, are you there?");
    });

    test('handle string parameters in a divert', () {
      story = loadStory('knot/param-strings.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(2);

      text.clear();
      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0], "\"I accuse myself!\" Poirot declared.");
    });

    test('handle passing integer as parameters in a divert', () {
      story = loadStory('knot/param-ints.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0], "You give 2 dollars.");
    });

    test('handle passing floats as parameters in a divert', () {
      story = loadStory('knot/param-floats.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0], "You give 2.5 dollars.");
    });

    test('handle passing variables as parameters in a divert', () {
      story = loadStory('knot/param-vars.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(1);

      text.clear();
      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0], "You give 2 dollars.");
    });

    test('handle passing multiple values as parameters in a divert', () {
      story = loadStory('knot/param-multi.ink.json');
      var text = <String>[];

      nextAll(story, text);
      story.chooseChoiceIndex(0);

      text.clear();
      nextAll(story, text);
      expect(text.length, 1);
      expect(text[0], "You give 1 or 2 dollars. Hmm.");
    });

    test('should support recursive calls with parameters on a knot', () {
      story = loadStory('knot/param-recurse.ink.json');
      var text = <String>[];

      nextAll(story, text);
      expect(text.length, 2);
      expect(text[0], "\"The result is 120!\" you announce.");
    });
  });
}
