import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  logSetUp();
  group('List Spec', () {
    late Story story;

    setUp(() {});
    tearDown(() {
      story.resetState();
    });
    test('test List Basic Operations', () {
      story = loadStory('lists/basic-operations.ink.json');

      expect(story.continueMaximally(),
          "b, d\na, b, c, e\nb, c\nfalse\ntrue\ntrue\n");
    });

    test('Test List Mixed Items', () {
      story = loadStory('lists/list-mixed-items.ink.json');
      expect("a, y, c\n", story.continueMaximally());
    });

    test('TestMoreListOperations', () {
      story = loadStory('lists/more-list-operations.ink.json');
      story.allowExternalFunctionFallbacks = true;
      expect(story.continueMaximally(), '1\nl\nn\nl, m\nn\n');
    });

    test('testEmptyListOrigin', () {
      story = loadStory('lists/empty-list-origin.ink.json');
      expect(story.continueMaximally(), 'a, b\n');
    });

    test('test list save load', () {
      story = loadStory('lists/list-save-load.ink.json');
      expect(story.continueMaximally(), 'a, x, c\n');
      Map<String, dynamic> savedState = story.state.toJson();

      // Compile new version of the story
      story = loadStory('lists/list-save-load.ink.json');

      // Load saved game
      story.state.LoadJsonObj(savedState);

      // TODO(amond): choosePathString
      story.choosePathString("elsewhere");

      // FIXME: This is the test from the C# impl. Is it correct?
      // expect('a, x, c, z\n', story.continueMaximally());
//		Assert.assertEquals("a, x, c, z\n", story.continueMaximally());

      //Assert.assertEquals("z\n", story.continueMaximally());
    });

    test('TestEmptyListOriginAfterAssignment', () {
      story = loadStory('lists/empty-list-origin-after-assignment.ink.json');
      expect(story.continueMaximally(), 'a, b, c\n');
    });

    test('test list range', () {
      story = loadStory('lists/list-range.ink.json');
      expect(story.continueMaximally(),
          'Pound, Pizza, Euro, Pasta, Dollar, Curry, Paella\nEuro, Pasta, Dollar, Curry\nTwo, Three, Four, Five, Six\nPizza, Pasta\n');
    });

    test('testBugAddingElement', () {
      story = loadStory('lists/bug-adding-element.ink.json');

      String s = story.continueMaximally();
      expect(s, '');

      story.chooseChoiceIndex(0);
      s = story.continueMaximally();
      expect(s, 'a\n');

      story.chooseChoiceIndex(1);
      s = story.continueMaximally();
      expect(s, 'OK\n');
    });

    test('testListRandom', () {
      story = loadStory('lists/list_random.ink.json');

      while (story.canContinue) {
        final result = story.Continue();
        expect(result == "B\n" || result == "C\n" || result == "D\n", isTrue);
      }
    });
  });
}
