import 'dart:convert';

import 'package:test/test.dart';

import '../../runtime_tests.dart';
import 'inkfiles/compiled/builtins/floor_ceiling_and_casts.ink.json.dart'
    as floor_ceiling_and_casts;
import 'inkfiles/compiled/builtins/read_count_across_callstack.ink.json.dart'
    as read_count_across_callstack;
import 'inkfiles/compiled/builtins/read_count_across_threads.ink.json.dart'
    as read_count_across_threads;
import 'inkfiles/compiled/builtins/read_count_dot_separated_path.ink.json.dart'
    as read_count_dot_separated_path;
import 'inkfiles/compiled/builtins/turns.ink.json.dart' as turns;
import 'inkfiles/compiled/builtins/turns_since.ink.json.dart' as turns_since;
import 'inkfiles/compiled/builtins/turns_since_nested.ink.json.dart'
    as turns_since_nested;
import 'inkfiles/compiled/builtins/turns_since_with_variable_target.ink.json.dart'
    as turns_since_with_variable_target;
import 'inkfiles/compiled/builtins/visit_counts_when_choosing.ink.json.dart'
    as visit_counts_when_choosing;

void main() {
  group('Builtins', () {
    test('tests choice count', () async {
      var story = await createStory(jsonEncode(floor_ceiling_and_casts.json));
      // TODO(amond): c#, js implementation
      // expect(story.ContinueMaximally(), '1\n1\n2\n0.6666667\n0\n1\n');
      expect(
          story.ContinueMaximally(), '1.0\n1\n2.0\n0.6666666666666666\n0\n1\n');
    });

    test('tests read count accross callstack', () async {
      var story =
          await createStory(jsonEncode(read_count_across_callstack.json));
      expect(story.ContinueMaximally(),
          '1) Seen first 1 times.\nIn second.\n2) Seen first 1 times.\n');
    });

    test('tests read count accross threads', () async {
      var story = await createStory(read_count_across_threads.jsonString);
      expect(story.ContinueMaximally(), '1\n1\n');
    });

    test('tests read count dot deperated path', () async {
      var story = await createStory(read_count_dot_separated_path.jsonString);
      expect(story.ContinueMaximally(), 'hi\nhi\nhi\n3\n');
    });

    test('tests nested turns since', () async {
      var story = await createStory(turns_since_nested.jsonString);

      expect(story.ContinueMaximally(), '-1 = -1\n');

      expect(story.currentChoices.length, 1);
      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(), 'stuff\n0 = 0\n');

      expect(story.currentChoices.length, 1);
      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(), 'more stuff\n1 = 1\n');
    });

    test('tests nested turns since', () async {
      var story =
          await createStory(turns_since_with_variable_target.jsonString);

      expect(story.ContinueMaximally(), '0\n0\n');

      expect(story.currentChoices.length, 1);
      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(), '1\n');
    });

    test('tests turns since', () async {
      var story = await createStory(turns_since.jsonString);

      expect(story.ContinueMaximally(), '-1\n0\n');

      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(), '1\n');

      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(), '2\n');
    });

    test('tests turns', () async {
      var story = await createStory(turns.jsonString);

      for (var i = 0; i < 10; i++) {
        expect(story.Continue(), '$i\n');
        story.ChooseChoiceIndex(0);
      }
    });

    test('tests visit counts when choosing', () async {
      var story = await createStory(visit_counts_when_choosing.jsonString);

      expect(story.state.VisitCountAtPathString('TestKnot'), 0);
      expect(story.state.VisitCountAtPathString('TestKnot2'), 0);

      story.ChoosePathString('TestKnot');

      expect(story.state.VisitCountAtPathString('TestKnot'), 1);
      expect(story.state.VisitCountAtPathString('TestKnot2'), 0);

      story.Continue();

      expect(story.state.VisitCountAtPathString('TestKnot'), 1);
      expect(story.state.VisitCountAtPathString('TestKnot2'), 0);

      story.ChooseChoiceIndex(0);

      expect(story.state.VisitCountAtPathString('TestKnot'), 1);
      expect(story.state.VisitCountAtPathString('TestKnot2'), 0);

      story.Continue();

      expect(story.state.VisitCountAtPathString('TestKnot'), 1);
      expect(story.state.VisitCountAtPathString('TestKnot2'), 1);
    });
  });
}
