import 'dart:convert';

import 'package:test/test.dart';

import '../../runtime_tests.dart';
import '../test_utils.dart';
import 'inkfiles/compiled/choices/choice_count.ink.json.dart' as choice_count;
import 'inkfiles/compiled/choices/choice_diverts_to_done.ink.dart'
    as choice_diverts_to_done;
import 'inkfiles/compiled/choices/choice_thread_forking.ink.json.dart'
    as choice_thread_forking;
import 'inkfiles/compiled/choices/choice_with_brackets_only.ink.json.dart'
    as choice_with_brackets_only;
import 'inkfiles/compiled/choices/conditional_choices.ink.json.dart'
    as conditional_choices;
import 'inkfiles/compiled/choices/default_choices.ink.json.dart'
    as default_choices;
import 'inkfiles/compiled/choices/default_simple_gather.ink.json.dart'
    as default_simple_gather;
import 'inkfiles/compiled/choices/fallback_choice_on_thread.ink.json.dart'
    as fallback_choice_on_thread;
import 'inkfiles/compiled/choices/gather_choice_same_line.ink.json.dart'
    as gather_choice_same_line;
import 'inkfiles/compiled/choices/has_read_on_choice.ink.json.dart'
    as has_read_on_choice;
import 'inkfiles/compiled/choices/logic_in_choices.ink.json.dart'
    as logic_in_choices;
import 'inkfiles/compiled/choices/non_text_in_choice_inner_content.ink.json.dart'
    as non_text_in_choice_inner_content;
import 'inkfiles/compiled/choices/once_only_choices_can_link_back_to_self.ink.json.dart'
    as once_only_choices_can_link_back_to_self;
import 'inkfiles/compiled/choices/once_only_choices_with_own_content.ink.json.dart'
    as once_only_choices_with_own_content;
import 'inkfiles/compiled/choices/should_not_gather_due_to_choice.ink.json.dart'
    as should_not_gather_due_to_choice;
import 'inkfiles/compiled/choices/sticky_choices_stay_sticky.ink.json.dart'
    as sticky_choices_stay_sticky;
import 'inkfiles/compiled/choices/various_default_choices.ink.json.dart'
    as various_default_choices;

void main() {
  group('Choices', () {
    test('tests choice count', () async {
      var story = await createStory(choice_count.json);
      expect(story.Continue(), '2\n');
    });

    test('tests choice divert to done', () async {
      var story = await createStory(choice_diverts_to_done.json);
      story.Continue();

      expect(story.currentChoices.length, 1);
      story.ChooseChoiceIndex(0);

      expect(story.Continue(), 'choice');
    });

    test('tests choice with brackets only', () async {
      var story = await createStory(choice_with_brackets_only.json);
      story.Continue();

      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'Option');
      story.ChooseChoiceIndex(0);

      expect(story.Continue(), 'Text\n');
    });

    test('tests choice thread forking', () async {
      var story = await createStory(choice_thread_forking.json);

      story.Continue();
      expect(story.currentChoices, isNotEmpty);
      var savedState = story.state.ToJson();
      var savedStateJson = story.state.toJson();
      expect(savedStateJson['flows'], isNotNull);
      expect(savedStateJson['flows']['DEFAULT_FLOW'], isNotNull);
      expect(savedStateJson['flows']['DEFAULT_FLOW']['currentChoices'],
          hasLength(1));

      story = await createStory(choice_thread_forking.json);
      story.state.LoadJson(savedState);

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();

      expect(story.hasWarning, false);
    });

    test('tests conditional choices', () async {
      var story = await createStory(conditional_choices.jsonString);

      story.ContinueMaximally();

      expect(story.currentChoices.length, 4);
      expect(story.currentChoices[0].text, 'one');
      expect(story.currentChoices[1].text, 'two');
      expect(story.currentChoices[2].text, 'three');
      expect(story.currentChoices[3].text, 'four');
    });

    test('tests default choices', () async {
      var story = await createStory(default_choices.jsonString);

      expect(story.Continue(), '');
      expect(story.currentChoices.length, 2);

      story.ChooseChoiceIndex(0);
      expect(story.Continue(), 'After choice\n');

      expect(story.currentChoices.length, 1);

      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(), 'After choice\nThis is default.\n');
    });

    test('tests default simple gather', () async {
      var story = await createStory(default_simple_gather.jsonString);

      expect(story.Continue(), 'x\n');
    });

    test('tests fallback choice on thread', () async {
      var story = await createStory(fallback_choice_on_thread.jsonString);

      expect(story.Continue(), 'Should be 1 not 0: 1.\n');
    });

    test('tests gather choice same line', () async {
      var story = await createStory(gather_choice_same_line.jsonString);

      story.Continue();
      expect(story.currentChoices[0].text, 'hello');

      story.ChooseChoiceIndex(0);
      story.Continue();

      expect(story.currentChoices[0].text, 'world');
    });

    test('tests has read on choice', () async {
      var story = await createStory(has_read_on_choice.jsonString);

      story.ContinueMaximally();
      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'visible choice');
    });

    test('tests logic in choices', () async {
      var story = await createStory(jsonEncode(logic_in_choices.json));

      story.ContinueMaximally();
      expect(story.currentChoices[0].text, "'Hello Joe, your name is Joe.'");
      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(),
          "'Hello Joe,' I said, knowing full well that his name was Joe.\n");
    });

    test('tests logic in choices v21', () async {
      var story = await createStory(await loadFromUrl('https://raw.githubusercontent.com/y-lohse/inkjs/v2.2.0/src/tests/inkfiles/compiled/choices/logic_in_choices.ink.json'));

      story.ContinueMaximally();
      expect(story.currentChoices[0].text, "'Hello Joe, your name is Joe.'");
      story.ChooseChoiceIndex(0);
      expect(story.ContinueMaximally(),
          "'Hello Joe,' I said, knowing full well that his name was Joe.\n");
    });

    test('tests non text in choice inner content', () async {
      var story =
          await createStory(jsonEncode(non_text_in_choice_inner_content.json));

      story.Continue();
      story.ChooseChoiceIndex(0);

      expect(story.Continue(), 'option text. Conditional bit. Next.\n');
    });

    test('tests test once only choices can link back to self', () async {
      var story = await createStory(
          jsonEncode(once_only_choices_can_link_back_to_self.json));

      story.ContinueMaximally();

      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'First choice');

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();

      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'Second choice');

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();

      expect(story.hasError, false);
    });

    test('tests test once only choices with own content', () async {
      var story = await createStory(
          jsonEncode(once_only_choices_with_own_content.json));

      story.ContinueMaximally();

      expect(story.currentChoices.length, 3);

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();

      expect(story.currentChoices.length, 2);

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();

      expect(story.currentChoices.length, 1);

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();

      expect(story.currentChoices.length, 0);
    });

    test('tests should not gather due to choice', () async {
      var story =
          await createStory(jsonEncode(should_not_gather_due_to_choice.json));

      story.ContinueMaximally();
      story.ChooseChoiceIndex(0);

      expect(story.ContinueMaximally(), 'opt\ntext\n');
    });

    test('tests sticky choices stay sticky', () async {
      var story =
          await createStory(jsonEncode(sticky_choices_stay_sticky.json));

      story.ContinueMaximally();
      expect(story.currentChoices.length, 2);

      story.ChooseChoiceIndex(0);
      story.ContinueMaximally();
      expect(story.currentChoices.length, 2);
    });

    test('tests various default choices', () async {
      var story = await createStory(jsonEncode(various_default_choices.json));

      expect(story.ContinueMaximally(), '1\n2\n3\n');
    });
  });
}
