import 'dart:convert';

import 'package:test/test.dart';

import '../../runtime_tests.dart';
import 'inkfiles/compiled/diverts/basic_tunnel.ink.json.dart' as basic_tunnel;
import 'inkfiles/compiled/diverts/compare_divert_targets.ink.json.dart'
    as compare_divert_targets;
import 'inkfiles/compiled/diverts/complex_tunnels.ink.json.dart'
    as complex_tunnels;
import 'inkfiles/compiled/diverts/divert_in_conditional.ink.json.dart'
    as divert_in_conditional;
import 'inkfiles/compiled/diverts/divert_targets_with_parameters.ink.json.dart'
    as divert_targets_with_parameters;
import 'inkfiles/compiled/diverts/divert_to_weave_points.ink.json.dart'
    as divert_to_weave_points;
import 'inkfiles/compiled/diverts/done_stops_thread.ink.json.dart'
    as done_stops_thread;
import 'inkfiles/compiled/diverts/path_to_self.ink.json.dart' as path_to_self;
import 'inkfiles/compiled/diverts/same_line_divert_is_inline.ink.json.dart'
    as same_line_divert_is_inline;
import 'inkfiles/compiled/diverts/tunnel_onwards_after_tunnel.ink.json.dart'
    as tunnel_onwards_after_tunnel;
import 'inkfiles/compiled/diverts/tunnel_onwards_divert_after_with_arg.ink.json.dart'
    as tunnel_onwards_divert_after_with_arg;
import 'inkfiles/compiled/diverts/tunnel_onwards_divert_override.ink.json.dart'
    as tunnel_onwards_divert_override;
import 'inkfiles/compiled/diverts/tunnel_onwards_with_param_default_choice.ink.json.dart'
    as tunnel_onwards_with_param_default_choice;
import 'inkfiles/compiled/diverts/tunnel_vs_thread_behaviour.ink.json.dart'
    as tunnel_vs_thread_behaviour;

void main() {
  group('Diverts', () {
    test('tests basic tunnel', () async {
      var story = await createStory(jsonEncode(basic_tunnel.json));

      expect(story.Continue(), 'Hello world\n');
    });

    test('tests compare divert targets', () async {
      var story = await createStory(jsonEncode(compare_divert_targets.json));

      expect(story.ContinueMaximally(),
          'different knot\nsame knot\nsame knot\ndifferent knot\nsame knot\nsame knot\n');
    });

    test('tests complex tunnels', () async {
      var story = await createStory(jsonEncode(complex_tunnels.json));

      expect(story.ContinueMaximally(),
          'one (1)\none and a half (1.5)\ntwo (2)\nthree (3)\n');
    });

    test('tests divert in conditional', () async {
      var story = await createStory(jsonEncode(divert_in_conditional.json));

      expect(story.ContinueMaximally(), '');
    });

    test('tests divert targets with parameters', () async {
      var story =
          await createStory(jsonEncode(divert_targets_with_parameters.json));

      expect(story.ContinueMaximally(), '5\n');
    });

    test('tests divert to weave points', () async {
      var story = await createStory(jsonEncode(divert_to_weave_points.json));

      expect(story.ContinueMaximally(),
          'gather\ntest\nchoice content\ngather\nsecond time round\n');
    });

    test('tests done stop thread', () async {
      var story = await createStory(jsonEncode(done_stops_thread.json));

      expect(story.ContinueMaximally(), '');
    });

    test('tests path to self', () async {
      var story = await createStory(jsonEncode(path_to_self.json));

      story.Continue();
      story.ChooseChoiceIndex(0);

      story.Continue();
      story.ChooseChoiceIndex(0);

      expect(story.canContinue, true);
    });

    test('tests same line divert is inline', () async {
      var story =
          await createStory(jsonEncode(same_line_divert_is_inline.json));

      expect(story.Continue(),
          'We hurried home to Savile Row as fast as we could.\n');
    });

    test('tests tunnel onwards after tunnel', () async {
      var story =
          await createStory(jsonEncode(tunnel_onwards_after_tunnel.json));

      expect(story.ContinueMaximally(), 'Hello...\n...world.\nThe End.\n');
    });

    test('tests tunnel onwards divert after with arg', () async {
      var story = await createStory(
          jsonEncode(tunnel_onwards_divert_after_with_arg.json));

      expect(story.ContinueMaximally(), '8\n');
    });

    test('tests tunnel onwards divert override', () async {
      var story =
          await createStory(jsonEncode(tunnel_onwards_divert_override.json));

      expect(story.ContinueMaximally(), 'This is A\nNow in B.\n');
    });

    test('tests tunnel onwardswith param default choice', () async {
      var story = await createStory(
          jsonEncode(tunnel_onwards_with_param_default_choice.json));

      expect(story.ContinueMaximally(), '8\n');
    });

    test('tests tunnel vs thread behaviour', () async {
      var story =
          await createStory(jsonEncode(tunnel_vs_thread_behaviour.json));

      expect(story.ContinueMaximally(), isNot('Finished tunnel'));
      expect(story.currentChoices.length, 2);

      story.ChooseChoiceIndex(0);

      expect(story.ContinueMaximally(), contains('Finished tunnel'));
      expect(story.currentChoices.length, 3);

      story.ChooseChoiceIndex(2);

      expect(story.ContinueMaximally(), contains('Done.'));
    });
  });
}
