import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../../../runtime_tests.dart';
import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

void main() {
  group('Inkjs', () {
    group('Content', () {
      late Story story;

      setUp(() async {
        story = await createStory(jsonEncode($tests.json));
      });

      test("should read simple content", () {
        story.ChoosePathString("content.simple");

        expect(story.Continue(), "Simple content inside a knot\n");
      });

      test("should read multiline content", () {
        story.ChoosePathString("content.multiline");

        expect(story.Continue(), "First line\n");
        expect(story.canContinue, isTrue);
        expect(story.Continue(), "Second line\n");
      });

      test("should print a variable", () {
        story.ChoosePathString("content.variable_text");

        expect(story.Continue(), "variable text\n");
      });

      test("should print a truthy conditional text", () {
        story.ChoosePathString("content.if_text_truthy");

        expect(story.Continue(), "I… I saw him. Only for a moment.\n");
      });

      test("should print a falsy conditional text", () {
        story.ChoosePathString("content.if_text_falsy");
        expect(story.Continue(), "I…\n");
      });

      test("should handle an if/else text", () {
        story.ChoosePathString("content.if_else_text");

        expect(story.Continue(), "I saw him. Only for a moment.\n");
        expect(story.Continue(), "I missed him. Was he particularly evil?\n");
      });
    });
  });
}
