import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../../../runtime_tests.dart';
import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

void main() {
  group('Lists', () {
    late Story story;

    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
      story.allowExternalFunctionFallbacks = true;
    });

    tearDown(() {});
    test('should be defined', () {
      story.ChoosePathString('lists.basic_list');

      expect(story.Continue(), 'cold\n');
      expect(story.Continue(), 'boiling\n');
    });

    test('should increment/decrement', () {
      story.ChoosePathString('lists.increment');

      expect(story.Continue(), 'cold\n');
      expect(story.Continue(), 'boiling\n');
      expect(story.Continue(), 'evaporated\n');
      expect(story.Continue(), 'boiling\n');
      expect(story.Continue(), 'cold\n');
    });

    test('should print the values', () {
      story.ChoosePathString('lists.list_value');

      expect(story.Continue(), '1\n');
      expect(story.Continue(), '2\n');
      expect(story.Continue(), '3\n');
    });

    test('should set names from values', () {
      story.ChoosePathString('lists.value_from_number');

      expect(story.Continue(), 'cold\n');
      expect(story.Continue(), 'boiling\n');
      expect(story.Continue(), 'evaporated\n');
    });

    test('should handle user defined values', () {
      story.ChoosePathString('lists.defined_value');
      expect(story.Continue(), '2\n');
      expect(story.Continue(), '3\n');

      // That's 0 and not 5, because it adds up to a non existing
      // list entry see https://github.com/inkle/ink/issues/441.
      expect(story.Continue(), '0\n');
    });

    test('should add and remove values from lists', () {
      story.ChoosePathString('lists.multivalue');
      expect(story.Continue(), '\n');
      expect(story.Continue(), 'Denver, Eamonn\n');
      expect(story.Continue(), 'Denver\n');
      expect(story.Continue(), '\n');
      expect(story.Continue(), '\n');
      expect(story.Continue(), 'Eamonn\n');
    });

    test('should resolve list queries', () {
      story.ChoosePathString('lists.listqueries');
      expect(story.Continue(), 'list is empty\n');
      expect(story.Continue(), '2\n');
      expect(story.Continue(), 'Denver\n');
      expect(story.Continue(), 'Eamonn\n');
      expect(story.Continue(), 'list is not empty\n');

      expect(story.Continue(), 'exact equality\n');
      expect(story.Continue(), 'falsy exact equality\n');
      expect(story.Continue(), 'exact inequality\n');
      expect(story.Continue(), 'exact inequality works\n');

      expect(story.Continue(), 'has Eamonn\n');
      expect(story.Continue(), 'has falsy works\n');
      expect(story.Continue(), 'has not\n');
      expect(story.Continue(), 'falsy has not\n');
      expect(story.Continue(), 'Adams, Bernard, Cartwright, Denver, Eamonn\n');
      expect(story.Continue(), '\n');
      expect(story.Continue(), '\n');

      expect(story.Continue(), 'truthy greater than\n');
      expect(story.Continue(), 'falsy greater than\n');
      expect(story.Continue(), 'greater than empty\n');
      expect(story.Continue(), 'empty greater than\n');

      expect(story.Continue(), 'truthy smaller than\n');
      expect(story.Continue(), 'falsy smaller than\n');
      expect(story.Continue(), 'smaller than empty\n');
      expect(story.Continue(), 'empty smaller than\n');

      expect(story.Continue(), 'truthy greater than or equal\n');
      expect(story.Continue(), 'truthy greater than or equal\n');
      expect(story.Continue(), 'falsy greater than or equal\n');
      expect(story.Continue(), 'greater than or equals empty\n');
      expect(story.Continue(), 'empty greater than or equals\n');

      expect(story.Continue(), 'truthy smaller than or equal\n');
      expect(story.Continue(), 'truthy smaller than or equal\n');
      expect(story.Continue(), 'falsy smaller than or equal\n');
      expect(story.Continue(), 'smaller than or equals empty\n');
      expect(story.Continue(), 'empty smaller than or equals\n');

      expect(story.Continue(), 'truthy list AND\n');
      expect(story.Continue(), 'falsy list AND\n');
      expect(story.Continue(), 'truthy list OR\n');
      expect(story.Continue(), 'falsy list OR\n');
      expect(story.Continue(), 'truthy list not\n');
      expect(story.Continue(), 'falsy list not\n');

      expect(story.Continue(), 'Bernard, Cartwright, Denver\n');
      expect(story.Continue(), 'Smith, Jones\n');

      expect(story.Continue(), 'Carter, Braithwaite\n');
      expect(story.Continue(), 'self_belief\n');
    });
  });
}
