import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../../../runtime_tests.dart';
import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

void main() {
  group('Simple Lists', () {
    late Story story;

    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
      story.allowExternalFunctionFallbacks = true;
    });
    test('should go through a sequence', () {
      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'one\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'two\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'three\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'final\n');

      story.ChoosePathString('simple_lists.sequence');
      expect(story.Continue(), 'final\n');
    });

    test('should go through a cycle', () {
      var results = ['one\n', 'two\n', 'three\n'];

      for (var i = 0; i < 10; ++i) {
        story.ChoosePathString('simple_lists.cycle');
        expect(story.Continue(), results[i % 3]);
      }
    });

    test('should go through a list once', () {
      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), 'one\n');

      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), 'two\n');

      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), 'three\n');

      story.ChoosePathString('simple_lists.once');
      expect(story.Continue(), '');
    });

    test('should go through a shuffle', () {
      var results = ['heads\n', 'tails\n'];

      for (var i = 0; i < 40; ++i) {
        story.ChoosePathString('simple_lists.shuffle');
        expect(results, contains(story.Continue()));
      }
    });

    test('should handle blank elements', () {
      for (var i = 0; i < 3; ++i) {
        story.ChoosePathString('simple_lists.blanks');
        expect(story.Continue(), '');
      }

      story.ChoosePathString('simple_lists.blanks');
      expect(story.Continue(), 'end\n');
    });
  });
}
