import 'dart:convert';

import 'package:ink_runtime/ink.dart';
import 'package:test/test.dart';

import '../../../runtime_tests.dart';
import '../inkfiles/compiled/inkjs/tests.ink.json.dart' as $tests;

void main() {
  group('Tags', () {
    late Story story;

    setUp(() async {
      story = await createStory(jsonEncode($tests.json));
      story.allowExternalFunctionFallbacks = true;
    });
    test('should find global tags', () {
      var tags = story.globalTags;

      expect(tags.length, 1);
      expect(tags[0], 'global tag');
    });

    test('should find knot level tags', () {
      var tags = story.TagsForContentAtPath('tags');

      expect(tags.length, 1);
      expect(tags[0], 'knot tag');
    });

    test('should find line by line tags', () {
      story.ChoosePathString('tags.line_by_Line');
      story.Continue();

      var tags = story.currentTags;

      expect(tags.length, 1);
      expect(tags[0], 'a tag');

      story.Continue();
      tags = story.currentTags;

      expect(tags.length, 2);
      expect(tags[0], 'tag1');
      expect(tags[1], 'tag2');

      story.Continue();
      tags = story.currentTags;

      expect(tags.length, 2);
      expect(tags[0], 'tag above');
      expect(tags[1], 'tag after');
    });

    test('should find tags on choice points', () {
      story.ChoosePathString('tags.choice');
      story.Continue();

      expect(story.currentChoices.length, 1);
      expect(story.currentChoices[0].text, 'a choice');
      expect(story.currentTags.length, 0);

      story.ChooseChoiceIndex(0);

      expect(story.Continue(), 'a choice\n');
      expect(story.currentTags.length, 1);
      expect(story.currentTags[0], 'a tag');
    });

    test('should handle tag edge cases', () {
      story.ChoosePathString('tags.weird');
      story.Continue();

      var tags = story.currentTags;

      expect(tags.length, 5);
      expect(tags[0], 'space around');
      expect(tags[1], '');
      expect(tags[2], '');
      expect(tags[3], '');
      expect(tags[4], '0');
    });
  });
}
