import 'package:test/test.dart';

import '../../runtime_tests.dart';
import '../test_utils.dart';

void main() {
  group('Tags', () {
    test('tests string constants', () async {
      var story = await createStory(await loadFromUrl(
          'https://raw.githubusercontent.com/y-lohse/inkjs/v2.2.0/src/tests/inkfiles/compiled/tags/tags.ink.json'));

      var globalTags = ['author: Joe', 'title: My Great Story'];
      var knotTags = ['knot tag'];
      var knotTagsWhenContinuedTwiceTags = ['end of knot tag'];
      var stitchTags = ['stitch tag'];

      expect(story.TagsAtStartOfFlowContainerWithPathString(''), globalTags);

      expect(story.globalTags, globalTags);
      expect(story.Continue(), 'This is the content\n');
      expect(story.currentTags, globalTags);

      expect(story.TagsForContentAtPath('knot'), knotTags);
      expect(story.TagsForContentAtPath('knot.stitch'), stitchTags);

      story.ChoosePathString('knot');
      expect(story.Continue(), 'Knot content\n');
      expect(story.currentTags, knotTags);
      expect(story.Continue(), '');
      expect(story.currentTags, knotTagsWhenContinuedTwiceTags);
    });

    test('tests tags in a sequence', () async {
      var story = await createStory(await loadFromUrl(
          'https://raw.githubusercontent.com/y-lohse/inkjs/v2.2.0/src/tests/inkfiles/compiled/tags/tags_in_seq.ink.json'));
      expect(story.Continue(),"A red sequence.\n");
      expect(story.currentTags,["red"]);

      expect(story.Continue(),"A white sequence.\n");
      expect(story.currentTags,["white"]);
    });

    test('tests dynamic content in tags', () async {
      var story = await createStory(await loadFromUrl(
          'https://raw.githubusercontent.com/y-lohse/inkjs/v2.2.0/src/tests/inkfiles/compiled/tags/tags_dynamic_content.ink.json'));
      expect(story.Continue(),"tag\n");
      expect(story.currentTags,["pic8red.jpg"]);
    });
  });
}
