import 'package:test/test.dart';

import 'test_utils.dart';

void main() {
  test('tag and content', () {
    var story = loadStory('tags/tags.ink.json');

    final globalTags = ['author: Joe', 'title: My Great Story'];
    final knotTags = ['knot tag'];
    final stitchTags = ['stitch tag'];
    final knotTagWhenContinuedTwice = ["end of knot tag"];

    expect(story.globalTags, containsAll(globalTags));

    expect(story.Continue(), 'This is the content\n');
    expect(story.currentTags, globalTags);

    expect(knotTags, story.TagsForContentAtPath("knot"));
    expect(stitchTags, story.TagsForContentAtPath("knot.stitch"));

    story.ChoosePathString("knot");

    expect(
      story.Continue(),
      "Knot content\n",
    );

    expect(story.currentTags, knotTags);

    expect(story.canContinue, true);

    expect(story.Continue(), '');

    expect(story.currentTags, knotTagWhenContinuedTwice);
  });

  test('Tags In Seq', () {
    var story = loadStory('tags/tagsInSeq.ink.json');

    expect(story.Continue(), 'A red sequence.\n');
    expect(story.currentTags, ['red']);

    expect(story.Continue(), 'A white sequence.\n');
    expect(story.currentTags, ['white']);
  });

  test('Tags In Choice', () {
    var story = loadStory('tags/tagsInChoice.ink.json');

    story.Continue();
    expect(story.currentTags.length, 0);
    expect(story.currentChoices.length, 1);

    expect(story.currentChoices.first.tags, ['one', 'two']);

    story.chooseChoiceIndex(0);
    expect(story.Continue(), "one three");
    expect(story.currentTags, ['one', 'three']);
  });

  test('testTagsDynamicContent', () {
    var story = loadStory('tags/tagsDynamicContent.ink.json');

    expect(story.Continue(), 'tag\n');
    expect(story.currentTags, ['pic8red.jpg']);
  });
}
