import 'package:ink_runtime/src/simple_json.dart';
import 'package:test/test.dart';

void main() {
  group('SimpleJsonWriter', () {
    late SimpleJsonWriter writer;

    setUp(() {
      writer = SimpleJsonWriter();
    });
    test("writes a proper inner hierarchy", () {
      writer.WriteObjectStart();
      writer.WriteProperty("callstackThreads", (writer) {
        writer.WriteObjectStart();
        {
          writer.WritePropertyStart("callstack");
          {
            writer.WriteArrayStart();
            {
              writer.WriteObjectStart();
              {
                writer.WriteProperty("cPath", "path.to.component");
                writer.WriteIntProperty("idx", 2);
                writer.WriteProperty("exp", "expression");
                writer.WriteIntProperty("type", 3);
              }
              writer.WriteObjectEnd();
              writer.WriteNull();
            }
            writer.WriteArrayEnd();
          }
          writer.WritePropertyEnd();

          writer.WriteIntProperty("threadIndex", 0);
          writer.WriteProperty("previousContentObject", "path.to.object");
        }
        writer.WriteObjectEnd();
      });

      writer.WriteIntProperty("inkSaveVersion", 8);
      writer.WriteObjectEnd();

      expect(writer.toString(),
          '{"callstackThreads":{"callstack":[{"cPath":"path.to.component","idx":2,"exp":"expression","type":3},null],"threadIndex":0,"previousContentObject":"path.to.object"},"inkSaveVersion":8}');
    });

    test("writes a proper inner string", () {
      writer.WriteObjectStart();
      {
        writer.WritePropertyNameStart();
        writer.WritePropertyNameInner("prop");
        writer.WritePropertyNameInner("erty");
        writer.WritePropertyNameEnd();

        writer.WriteStringStart();
        writer.WriteStringInner("^");
        writer.WriteStringInner("Hello World.");
        writer.WriteStringEnd();
        writer.WritePropertyEnd();

        writer.WritePropertyStart("key");
        writer.WriteArrayStart();
        {
          writer.WriteStringStart();
          writer.WriteStringInner("^");
          writer.WriteStringInner("Hello World.");
          writer.WriteStringEnd();
        }
        writer.WriteArrayEnd();
        writer.WritePropertyEnd();
      }
      writer.WriteObjectEnd();

      expect(writer.toString(),
          '{"property":"^Hello World.","key":["^Hello World."]}');
    });

    test("handles nested arrays", () {
      writer.WriteArrayStart();
      {
        writer.WriteArrayStart();
        {
          writer.WriteArrayStart();
          {
            writer.WriteArrayStart();
            writer.WriteNull();
            writer.WriteArrayEnd();
          }
          writer.WriteArrayEnd();
        }
        writer.WriteArrayEnd();
      }
      writer.WriteArrayEnd();

      expect(writer.toString(), "[[[[null]]]]");
    });

    test("throws with unbalanced calls", () {
      expect(() {
        writer.WriteObjectStart();
        writer.WritePropertyEnd();
      }, throwsA(anything));

      expect(() {
        writer.WriteStringStart();
        writer.WriteArrayStart();
        writer.WriteStringEnd();
      }, throwsA(anything));
    });

    group("when writing integers", () {
      test("creates the proper object hierarchy", () {
        writer.WriteObjectStart();
        writer.WriteIntProperty("property", 3);
        writer.WriteObjectEnd();

        expect(writer.toString(), '{"property":3}');
      });

      test("creates the proper object hierarchy", () {
        writer.WriteArrayStart();
        writer.WriteInt(3);
        writer.WriteArrayEnd();

        expect(writer.toString(), "[3]");
      });

      /*
      test("converts floats into integer", () {
        writer.WriteArrayStart();
        {
          writer.WriteObjectStart();
          writer.WriteIntProperty("property", 3.9);
          writer.WriteObjectEnd();

          writer.WriteArrayStart();
          writer.WriteInt(3.1);
          writer.WriteInt(4.0);
          writer.WriteArrayEnd();
        }
        writer.WriteArrayEnd();

        expect(writer.toString(), '[{"property":3},[3,4]]');
      });*/
    });

    group("when writing floats", () {
      test("creates the proper object hierarchy", () {
        writer.WriteObjectStart();
        writer.WriteFloatProperty("property", 3.4);
        writer.WriteObjectEnd();

        expect(writer.toString(), '{"property":3.4}');
      });

      test("creates the proper object hierarchy", () {
        writer.WriteArrayStart();
        writer.WriteFloat(36.1456);
        writer.WriteArrayEnd();

        expect(writer.toString(), "[36.1456]");
      });

      /*
      test("doesn't converts integer into floats", () {
        writer.WriteArrayStart();
        writer.WriteFloat(3);
        writer.WriteFloat(4);
        writer.WriteArrayEnd();

        expect(writer.toString(), "[3,4]");
      });*/

      test("converts infinity and NaN", () {
        writer.WriteArrayStart();
        writer.WriteFloat(double.infinity);
        writer.WriteFloat(double.negativeInfinity);
        writer.WriteFloat(double.nan);
        writer.WriteArrayEnd();

        expect(writer.toString(), "[3.4e+38,-3.4e+38,0.0]");
      });
    });
  });
  group("SimpleJson.Reader", () {
    test("parses a JSON object string", () {
      var jsonString = '{"key":"value", "array": [1, 2, null, 3.0, false]}';
      var object = {
        'array': [1, 2, null, 3.0, false],
        'key': "value",
      };

      var reader = SimpleJsonReader(jsonString);

      expect(reader.ToDictionary(), object);
      expect(SimpleJson.TextToDictionary(jsonString), object);
    });

    test("parses a JSON array string", () {
      var jsonString = "[1, 2, null, 3.0, false]";
      var object = [1, 2, null, 3.0, false];

      var reader = SimpleJsonReader(jsonString);

      expect(reader.ToArray(), object);
      expect(SimpleJson.TextToArray(jsonString), object);
    });

    test("throws if the json is malformed", () {
      var jsonString = '{key: "value"]';

      expect(() {
        var reader = SimpleJsonReader(jsonString);
        reader.ToDictionary();
      }, throwsA(anything));

      expect(() {
        SimpleJson.TextToDictionary(jsonString);
      }, throwsA(anything));
    });
  });
}
